/* 

		Routinen zur Steuerung des I2C-Bus am Printerport des ST
		(c) 1993 Till Harbaum

*/

#include <tos.h>
#include "videotxt.h"

#define giselect (*(char*)0xffff8800L)
#define giwrite  (*(char*)0xffff8802L)
#define CENTRONICS giwrite
#define BUSY (*(char*)0xffffFA01L & 1)
#define flock ( *(int*) 0x43e )

int	SDA_Shadow ;	/* speichert letzten geschriebenen Wert */
int prt_adap;			/* spezial Printer-Adapter ist angeschlossen */

/* SDA wird auf der Platine durch T1 invertiert */
int olli=1;

void SDA_Low (void)
{
	if(!olli)
		SDA_Shadow &= 0x00fe ;
	else
		SDA_Shadow |= 0x001 ;
	
	CENTRONICS = SDA_Shadow ;
}

void SDA_High (void)
{
	if(!olli)
		SDA_Shadow |= 0x001 ;
	else
		SDA_Shadow &= 0x00fe ;
	
	CENTRONICS = SDA_Shadow ;
}

void SCL_Low (void)
{
	SDA_Shadow &= 0x00fd ;
	CENTRONICS = SDA_Shadow ;
}

void SCL_High (void)
{
	SDA_Shadow |= 0x002 ;
	CENTRONICS = SDA_Shadow ;
}

void  starti2c (void)  /* Start fr I2C-Bus */
{
	char buff;

  /* evtl. Drucker abschalten */
	if(prt_adap)
	{
		CENTRONICS=0;		/* Nullbyte senden */
	  giselect=0x0e;  /* Port A */
	  buff=giselect;  /* Leseport ist Select-Register */
	  buff&=~32;			/* STROBE */
	  CENTRONICS=buff;
	  giselect=0x0f;	/* und wieder Port B */
	  SCL_Low();
	  SCL_High();
	}

	SDA_Low() ;
	SCL_Low() ;
}

void  stopi2c (void)   /* Ende der bertragung */
{
	char buff;

	SDA_Low() ;
	SCL_High() ;
	SDA_High() ;

  /* evtl. Drucker wieder anschalten */
	if(prt_adap)
	{
	  giselect=0x0e;  /* Port A */
	  buff=giselect;  /* Leseport ist Select-Register */
	  buff|=32;			  /* STROBE wieder hi */
	  CENTRONICS=buff;
	  giselect=0x0f;
	}
} ;


#define SuperON {ssp=Super ((void*)0L); flock=1; giselect = 0xf; }
#define SuperOFF {flock=0; 	Super ((void*) ssp); }

void reseti2c (void)
{
	long	ssp ;

	SuperON ;
	stopi2c () ;
	SuperOFF ;
}

int sendbytei2c(char daten)
{
	int		x ;
	
	for (x=7; x>=0; x--)
	{
		if ( daten & 0x80 )
    		SDA_High() ;
    	else 
    		SDA_Low() ;
    		
		SCL_High() ;
		SCL_Low() ;
		
		daten <<= 1 ;
	}

  SDA_High() ;
  SCL_High() ;
  	
  x = BUSY ;
	SCL_Low() ;

	if ( x )
		return 0 ;
	else
		return 1 ;
}

int test_i2c(int addr)
{
	int a,b,ok=0;
	char buff;
	long ssp;

	SuperON;

	/* BUSY muss mit SDA mitgehen */
	SDA_High();
	a= BUSY;
	SDA_Low();
	b= BUSY;

	/* Kann der Adpater schon so angesprochen werden ?? */
	prt_adap=(a==b);

  /* evtl. Drucker abschalten */
	if(prt_adap)
	{
		CENTRONICS=0;		/* Nullbyte senden */
 	 	giselect=0x0e;  /* Port A */
 	 	buff=giselect;  /* Leseport ist Select-Register */
	  buff&=~32;			/* STROBE */
	  CENTRONICS=buff;
	  giselect=0x0f;	/* und wieder Port B */
  
	  SCL_Low();			/* Flanke zum Umschalten geben */
	  SCL_High();
	}
  
	/* BUSY muss mit SDA mitgehen */
	SDA_High();
	a= BUSY;
	SDA_Low();
	b= BUSY;

  /* evtl. Drucker wieder anschalten */
	if(prt_adap)
	{
	  giselect=0x0e;  /* Port A */
	  buff=giselect;  /* Leseport ist Select-Register */
	  buff|=32;			  /* STROBE wieder hi */
	  CENTRONICS=buff;
	  giselect=0x0f;
	}
	
	if(a==b)
		ok=1;
		
	if(ok==0)
	{
		/* gucken, ob kein c't-Adapter dran */
		if(a==0)
			olli=0;

		stopi2c();
		starti2c();
		if(!sendbytei2c(addr))
			ok=-1;
		stopi2c();
	}

	SuperOFF;
	
	return(ok);
}

int sendi2c(unsigned char addr, int anzahl, unsigned char *daten)
{
	int 	x ;
	unsigned char	*s ;
	long	ssp ;

	SuperON ;

	starti2c() ;
 
	if ( ! sendbytei2c(addr))
	{
		stopi2c() ;
		SuperOFF ;
    	return 0 ;	/*Fehler*/
	}
	
	s = daten ;
 	for (x=0; x<anzahl; x++)
	{
    	if ( ! sendbytei2c(*s++))
		{
			stopi2c() ;
			SuperOFF ;
    		return 0 ;	/*Fehler*/
		}
  } 
	stopi2c() ;

	SuperOFF ;
	return 1 ;
}

int receivebytei2c (unsigned char *daten, int letztes)
{
	int x, c ;
	
	SDA_High() ;  /* fr lesen */  
	SCL_Low() ;
	
	c = 0 ;
	for (x=7; x>=0; x--)
	{
   	SCL_High() ; 
    	
   	c += (c + BUSY);
    SCL_Low() ;
 	}
  	
 	/* Quittierungsbit senden */
 	if ( letztes ) 
 		SDA_High() ;
 	else 
 		SDA_Low() ;
  		
 	SCL_High() ;  
 	SCL_Low() ;  
 	SDA_High() ;
  	
 	*daten = c ;
	
	return 1 ;
}

int receivei2c(unsigned char addr, int anzahl, unsigned char *daten)
{
	int		i ;
	unsigned char	*s ;
	long	ssp ;

	SuperON ;

	starti2c() ;

 	/* Bausteinadresse schreiben */
 	if ( ! sendbytei2c(addr))
	{
		stopi2c() ;
		SuperOFF ;
    	return(0) ;	/*Fehler*/
	}
	
	SDA_High() ; /* fr lesen */

	s = daten ;
	for (i=anzahl; i>0; i--)
	{
	    receivebytei2c(s, (i==1) ) ;
	    s++ ;
 	}
	stopi2c() ;

	SuperOFF ;
	return(1) ;
}

