/*
 			RTC.C 						(c) 1994 Till Harbaum
 			
*/

#include <stdlib.h>
#include <stdio.h>
#include "rtc.h"

#define  RTCREAD      0xA1
#define  RTCWRITE     0xA0

#define  HEX2DEC(a)   (((a)&0xf)+10*((a)>>4))
#define  DEC2HEX(a)		(((a)%10)+16*((a)/10))

/* Konstanten fr Register-Nummern */
#define  RTC_CONTROL   0x00
#define  RTC_SEC100    0x01
#define  RTC_SECOND    0x02
#define  RTC_MINUTE    0x03
#define  RTC_HOUR      0x04
#define  RTC_YEARDATE  0x05
#define  RTC_WDAYMON   0x06
#define  RTC_TIMER     0x07
#define  RTC_ACONTROL  0x08
#define  RTC_ASEC100   0x09
#define  RTC_ASECOND   0x0A
#define  RTC_AMINUTE   0x0B
#define  RTC_AHOUR     0x0C
#define  RTC_ADATE     0x0D
#define  RTC_AMONTH    0x0E
#define  RTC_ATIMER    0x0F

#define uchar unsigned char

void show_reg(void);

uchar rtc_read_byte (char adr)
{
	uchar i2c_buff[2];

  i2c_buff[0] = adr;
  sendi2c(RTCWRITE, 1, i2c_buff);
  receivei2c(RTCREAD, 1, i2c_buff);

  return (i2c_buff[0]);
}

void rtc_write_byte (uchar adr, uchar byte)
{
  uchar i2c_buff[2];

  i2c_buff[0] = adr;
  i2c_buff[1] = byte;

  sendi2c(RTCWRITE, 2, i2c_buff);
}

void rtc_read_buff (uchar adr, uchar anzahl, uchar *buffer)
{
  sendi2c( RTCWRITE, 1, &adr);
  receivei2c( RTCREAD, anzahl, buffer);
}

void rtc_write_buff (uchar adr, uchar anzahl, uchar *buffer)
{
  uchar i,i2c_buff[20];

  for (i = anzahl; i; i--) i2c_buff[i] = buffer [i-1];

  i2c_buff[0] = adr;
  sendi2c(RTCWRITE, anzahl + 1, i2c_buff);
}

void rtc_init (void)
{
  /* -- Modus einstellen: Zhler, 32-kHz-Modus, alles lesbar, Alarm mgl. -- */
  rtc_write_byte (RTC_CONTROL, 0x00);

  /* --  24-Stunden-Modus setzen  */
  rtc_write_byte (RTC_HOUR, rtc_read_byte (RTC_HOUR) & 0x3F);

  /* --  keine Interrupts von Alarm oder Timer zulassen  */
  rtc_write_byte (RTC_ACONTROL, 0x00);
}

void rtc_reset (void)
{
  uchar count, i2c_buff[20];

  for (count = 0; count < 16; i2c_buff [count++] = 0 );
  rtc_write_buff (RTC_CONTROL, 16, i2c_buff);
}

void rtc_start (void)
{
  rtc_write_byte (RTC_CONTROL, 0x04);
}

void rtc_stop (void)
{
  rtc_write_byte (RTC_CONTROL, 0xC4);
}

int init_rtc(void)
{
	int 	test,ret=0;

	test=test_i2c(RTCWRITE);

	switch(test)
	{
		case 1:  	ret = -1;
							break;
		case -1:  ret = -1;
							break;
	}

	return(ret);
}

int  get_time(int *jahr, int *monat, int *tag, int *std, int *min, int *sec)
{
  uchar buffer [5];

  rtc_read_buff (RTC_SECOND, 5, buffer);
	*sec=HEX2DEC(buffer[0]);
	*min=HEX2DEC(buffer[1]);
	*std=HEX2DEC(buffer[2]&0x3f);
	*tag=HEX2DEC(buffer[3]&0x3f);
	*monat=HEX2DEC(buffer[4]&0x1f);
	*jahr=1994+((buffer[3]>>6)&3);

	return(0);
}

int set_time(int jahr, int monat, int tag, int std, int min, int sec)
{
  uchar buffer [6];

	buffer[0]=0;		
	buffer[1]=DEC2HEX(sec);
	buffer[2]=DEC2HEX(min);
	buffer[3]=DEC2HEX(std);
	buffer[4]=DEC2HEX(tag)|((jahr-1994)<<6);
	buffer[5]=DEC2HEX(monat);
	
	rtc_stop();
  rtc_write_buff (RTC_SEC100, 6, buffer);
  rtc_start();

	return(0);
}